@@ -2,7 +2,7 @@
 /*
 
 Copyright (C) 1993, 1994, 1995, 1996, 1997, 2000, 2002, 2003, 2004,
-              2005, 2006, 2007 John W. Eaton 
+              2005, 2006, 2007, 2008 John W. Eaton 
 
 This file is part of Octave.
 
@@ -63,6 +63,28 @@ Array<T>::~Array (void)
   delete [] idx;
 }
 
+template <class T>
+Array<T>&
+Array<T>::operator = (const Array<T>& a)
+{
+  if (this != &a)
+    {
+      if (--rep->count <= 0)
+	delete rep;
+
+      rep = a.rep;
+      rep->count++;
+
+      dimensions = a.dimensions;
+
+      delete [] idx;
+      idx_count = 0;
+      idx = 0;
+    }
+
+  return *this;
+}
+
 template <class T>
 Array<T>
 Array<T>::squeeze (void) const
@@ -2246,7 +2268,7 @@ Array<T>::indexN (idx_vector& ra_idx, int resize_ok, const T& rfv) const
 	{
 	  dim_vector result_dims;
 
-	  if (vec_equiv)
+	  if (vec_equiv && ! orig_len == 1)
 	    {
 	      result_dims = dv;
 
@@ -2392,7 +2414,7 @@ Array<T>::index (Array<idx_vector>& ra_idx, int resize_ok, const T& rfv) const
   dim_vector new_dims = dims ();
   dim_vector frozen_lengths;
 
-  if (! any_orig_empty (ra_idx) && ra_idx_len < n_dims)
+  if (!ra_idx (ra_idx_len - 1).orig_empty () && ra_idx_len < n_dims)
     frozen_lengths = short_freeze (ra_idx, dimensions, resize_ok);
   else
     {
@@ -3092,35 +3114,100 @@ assignN (Array<LT>& lhs, const Array<RT>& rhs, const LT& rfv)
 
       if (orig_empty)
 	{
-	  int k = 0;
-	  for (int i = 0; i < n_idx; i++)
+	  if (rhs_is_scalar)
 	    {
-	      // If index is a colon, resizing to RHS dimensions is
-	      // allowed because we started out empty.
-
-	      if (idx(i).is_colon ())
+	      for (int i = 0; i < n_idx; i++)
 		{
-		  if (k < rhs_dims.length ())
-		    new_dims(i) = rhs_dims(k++);
-		  else
+		  if (idx(i).is_colon ())
 		    new_dims(i) = 1;
+		  else
+		    new_dims(i) = idx(i).orig_empty () ? 0 : idx(i).max () + 1;
+		}
+	    }
+	  else if (is_vector (rhs_dims))
+	    {
+	      int ncolon = 0;
+	      int fcolon = 0;
+	      octave_idx_type new_dims_numel = 1;
+	      int new_dims_vec = 0;
+	      for (int i = 0; i < n_idx; i++)
+		if (idx(i).is_colon ())
+		  {
+		    ncolon ++;
+		    if (ncolon == 1)
+		      fcolon = i;
+		  } 
+		else
+		  {
+		    octave_idx_type cap = idx(i).capacity ();
+		    new_dims_numel *= cap;
+		    if (cap != 1)
+		      new_dims_vec ++;
+		  }
+
+	      if (ncolon == n_idx)
+		{
+		  new_dims = rhs_dims;
+		  new_dims.resize (n_idx);
+		  for (int i = rhs_dims_len; i < n_idx; i++)
+		    new_dims (i) = 1;
 		}
 	      else
 		{
-		  octave_idx_type nelem = idx(i).capacity ();
+		  octave_idx_type rhs_dims_numel = rhs_dims.numel ();
+	      	      
+		  for (int i = 0; i < n_idx; i++)
+		    new_dims(i) = idx(i).orig_empty () ? 0 : idx(i).max () + 1;
 
-		  if (nelem >= 1
-		      && ((k < rhs_dims.length () && nelem == rhs_dims(k))
-			  || rhs_is_scalar))
-		    k++;
-		  else if (! (nelem == 1 || rhs_is_scalar))
+		  if (new_dims_numel != rhs_dims_numel && 
+		      ncolon > 0 && new_dims_numel == 1)
+		    {
+		      if (ncolon == rhs_dims_len)
+			{
+			  int k = 0;
+			  for (int i = 0; i < n_idx; i++)
+			    if (idx(i).is_colon ())
+			      new_dims (i) = rhs_dims (k++);
+			}
+		      else
+			new_dims (fcolon) = rhs_dims_numel;
+		    }
+		  else if (new_dims_numel != rhs_dims_numel || new_dims_vec > 1)
 		    {
 		      (*current_liboctave_error_handler)
 			("A(IDX-LIST) = RHS: mismatched index and RHS dimension");
 		      return retval;
 		    }
+		}
+	    }
+	  else
+	    {
+	      int k = 0;
+	      for (int i = 0; i < n_idx; i++)
+		{
+		  if (idx(i).is_colon ())
+		    {
+		      if (k < rhs_dims_len)
+			new_dims(i) = rhs_dims(k++);
+		      else
+			new_dims(i) = 1;
+		    }
+		  else
+		    {
+		      octave_idx_type nelem = idx(i).capacity ();
 
-		  new_dims(i) = idx(i).orig_empty () ? 0 : idx(i).max () + 1;
+		      if (nelem >= 1 
+			  && (k < rhs_dims_len && nelem == rhs_dims(k)))
+			k++;
+		      else if (nelem != 1)
+			{
+			  (*current_liboctave_error_handler)
+			    ("A(IDX-LIST) = RHS: mismatched index and RHS dimension");
+			  return retval;
+			}
+		      new_dims(i) = idx(i).orig_empty () ? 0 : 
+			idx(i).max () + 1;
+		    }
 		}
 	    }
 	}
