@@ -1,14 +1,15 @@
 // Template array classes
 /*
 
-Copyright (C) 2000 John W. Eaton
+Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
+              John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -16,9 +17,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -29,12 +29,12 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 #include <cassert>
 #include <climits>
-#include <cmath>
 #include <cstdlib>
 
 #include "Array.h"
 #include "Array2.h"
 #include "lo-error.h"
+#include "lo-math.h"
 
 class idx_vector;
 
@@ -87,10 +87,10 @@ public:
   ArrayN<T> reshape (const dim_vector& new_dims) const
     { return Array<T>::reshape (new_dims); }
 
-  ArrayN<T> permute (const Array<int>& vec, bool inv = false) const
+  ArrayN<T> permute (const Array<octave_idx_type>& vec, bool inv = false) const
     { return Array<T>::permute (vec, inv); }
 
-  ArrayN<T> ipermute (const Array<int>& vec) const
+  ArrayN<T> ipermute (const Array<octave_idx_type>& vec) const
     { return Array<T>::ipermute (vec); }
 
   void resize (const dim_vector& dv)
