@@ -1,4 +1,4 @@
-## Copyright (C) 2006, 2007 John W. Eaton
+## Copyright (C) 2006, 2007, 2008 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -23,8 +23,17 @@
 ## parameter @var{shape}.
 ## @end deftypefn
 
+## Deprecated in version 3.0
+
 function inv = weibinv (varargin)
 
+  persistent warned = false;
+  if (! warned)
+    warned = true;
+    warning ("Octave:deprecated-function",
+             "weibinv is obsolete and will be removed from a future version of Octave; please use wblinv instead");
+  endif
+
   inv = wblinv (varargin{:});
 
 endfunction
