@@ -1,11 +1,12 @@
-## Copyright (C) 1996, 1997 John W. Eaton
+## Copyright (C) 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2004, 2005,
+##               2006, 2007 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
 ## Octave is free software; you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2, or (at your option)
-## any later version.
+## the Free Software Foundation; either version 3 of the License, or (at
+## your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +14,8 @@
 ## General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with Octave; see the file COPYING.  If not, write to the Free
-## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-## 02110-1301, USA.
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {} fliplr (@var{x})
@@ -30,18 +30,17 @@
 ## @end group
 ## @end example
 ##
-## Note that @code{fliplr} only workw with 2-D arrays.  To flip N-d arrays
+## Note that @code{fliplr} only work with 2-D arrays.  To flip N-d arrays
 ## use @code{flipdim} instead.
+## @seealso{flipud, flipdim, rot90, rotdim}
 ## @end deftypefn
-##
-## @seealso{flipud, flipdim, rot90 and rotdim}
 
 ## Author: jwe
 
 function y = fliplr (x)
 
   if (nargin != 1)
-    usage ("fliplr (x)");
+    print_usage ();
   endif
 
   if (ndims (x) > 2)
