@@ -40,14 +40,15 @@ along with Octave; see the file COPYING.  If not, see
 #include "quit.h"
 #include "parse.h"
 
-#if defined (HAVE_PCRE_PCRE_H)
-#include <pcre/pcre.h>
-#elif defined (HAVE_PCRE_H)
+#if defined (HAVE_PCRE)
 #include <pcre.h>
 #elif defined (HAVE_REGEX)
 #if defined (__MINGW32__)
 #define __restrict
 #endif
+#if defined (HAVE_SYS_TYPES_H)
+#include <sys/types.h>
+#endif
 #include <regex.h>
 #endif
 
@@ -82,17 +83,17 @@ typedef std::list<regexp_elem>::const_iterator const_iterator;
 static int
 octregexp_list (const octave_value_list &args, const std::string &nm, 
 		bool case_insensitive, std::list<regexp_elem> &lst, 
-		string_vector &named, int &nopts)
+		string_vector &named, int &nopts, bool &once)
 {
   int sz = 0;
 #if defined (HAVE_REGEX) || defined (HAVE_PCRE) 
   int nargin = args.length();
-  bool once = false;
   bool lineanchors = false;
   bool dotexceptnewline = false;
   bool freespacing = false;
 
   nopts = nargin - 2;
+  once = false;
 
   std::string buffer = args(0).string_value ();
   if (error_state)
@@ -450,7 +451,8 @@ octregexp (const octave_value_list &args, int nargout, const std::string &nm,
   std::list<regexp_elem> lst;
   string_vector named;
   int nopts;
-  int sz = octregexp_list (args, nm, case_insensitive, lst, named, nopts);
+  bool once;
+  int sz = octregexp_list (args, nm, case_insensitive, lst, named, nopts, once);
 
   if (! error_state)
     {
@@ -481,36 +483,70 @@ octregexp (const octave_value_list &args, int nargout, const std::string &nm,
       retval(5) = Octave_map();
 #endif
 
-      Cell t (dim_vector(1, sz));
-      i = 0;
-      for (const_iterator p = lst.begin(); p != lst.end(); p++)
-	t(i++) = p->t;
-      retval(4) = t;
+      if (once)
+        retval(4) = sz ? lst.front ().t : Cell();
+      else
+        {
+          Cell t (dim_vector(1, sz));
+          i = 0;
+          for (const_iterator p = lst.begin(); p != lst.end(); p++)
+            t(i++) = p->t;
+          retval(4) = t;
+        }
 
-      Cell m (dim_vector(1, sz));
-      i = 0;
-      for (const_iterator p = lst.begin(); p != lst.end(); p++)
-	m(i++) = p->m;
-      retval(3) = m;
+      if (once)
+        retval(3) = sz ? lst.front ().m : std::string();
+      else
+        {
+          Cell m (dim_vector(1, sz));
+          i = 0;
+          for (const_iterator p = lst.begin(); p != lst.end(); p++)
+            m(i++) = p->m;
+          retval(3) = m;
+        }
 
+      if (once)
+        retval(2) = sz ? lst.front ().te : Matrix();
+      else
+        {
+          Cell te (dim_vector(1, sz));
+          i = 0;
+          for (const_iterator p = lst.begin(); p != lst.end(); p++)
+            te(i++) = p->te;
+          retval(2) = te;
+        }
 
-      Cell te (dim_vector(1, sz));
-      i = 0;
-      for (const_iterator p = lst.begin(); p != lst.end(); p++)
-	te(i++) = p->te;
-      retval(2) = te;
-
-      NDArray e (dim_vector(1, sz));
-      i = 0;
-      for (const_iterator p = lst.begin(); p != lst.end(); p++)
-	e(i++) = p->e;
-      retval(1) = e;
+      if (once)
+        {
+          if (sz)
+            retval(1) = lst.front ().e;
+          else
+            retval(1) = Matrix();
+        }
+      else
+        {
+          NDArray e (dim_vector(1, sz));
+          i = 0;
+          for (const_iterator p = lst.begin(); p != lst.end(); p++)
+            e(i++) = p->e;
+          retval(1) = e;
+        }
 
+      if (once)
+        {
+          if (sz)
+            retval(0) = lst.front ().s;
+          else
+            retval(0) = Matrix();
+        }
+      else
+        {
       NDArray s (dim_vector(1, sz));
       i = 0;
       for (const_iterator p = lst.begin(); p != lst.end(); p++)
 	s(i++) = p->s;
       retval(0) = s;
+        }
 
       // Alter the order of the output arguments
       if (nopts > 0)
@@ -910,111 +946,93 @@ The pattern is taken literally.\n\
 %! [s, e, te, m, t] = regexp('short test string','\w*r\w*','once');
 %! assert (s,1)
 %! assert (e,5)
-%! assert (size(te), [1,1])
-%! assert (isempty(te{1}))
-%! assert (m{1},'short')
-%! ## Matlab gives [1,0] here but that seems wrong.
-%! assert (size(t), [1,1])
+%! assert (isempty(te))
+%! assert (m,'short')
+%! assert (isempty(t))
 
 %!test
 %! [m, te, e, s, t] = regexp('short test string','\w*r\w*','once', 'match', 'tokenExtents', 'end', 'start', 'tokens');
 %! assert (s,1)
 %! assert (e,5)
+%! assert (isempty(te))
+%! assert (m,'short')
+%! assert (isempty(t))
+
+%!testif HAVE_PCRE
+%! ## This test is expected to fail if PCRE is not installed
+%! [s, e, te, m, t, nm] = regexp('short test string','(?<word1>\w*t)\s*(?<word2>\w*t)');
+%! assert (s,1)
+%! assert (e,10)
 %! assert (size(te), [1,1])
-%! assert (isempty(te{1}))
-%! assert (m{1},'short')
-%! ## Matlab gives [1,0] here but that seems wrong.
-%! assert (size(t), [1,1])
+%! assert (te{1}, [1 5; 7, 10])
+%! assert (m{1},'short test')
+%! assert (size(t),[1,1])
+%! assert (t{1}{1},'short')
+%! assert (t{1}{2},'test')
+%! assert (size(nm), [1,1])
+%! assert (!isempty(fieldnames(nm)))
+%! assert (sort(fieldnames(nm)),{'word1';'word2'})
+%! assert (nm.word1,'short')
+%! assert (nm.word2,'test')
 
-%!test
+%!testif HAVE_PCRE
 %! ## This test is expected to fail if PCRE is not installed
-%! if (!isempty(findstr(octave_config_info ("DEFS"),"HAVE_PCRE")))
-%!   [s, e, te, m, t, nm] = regexp('short test string','(?<word1>\w*t)\s*(?<word2>\w*t)');
-%!   assert (s,1)
-%!   assert (e,10)
-%!   assert (size(te), [1,1])
-%!   assert (te{1}, [1 5; 7, 10])
-%!   assert (m{1},'short test')
-%!   assert (size(t),[1,1])
-%!   assert (t{1}{1},'short')
-%!   assert (t{1}{2},'test')
-%!   assert (size(nm), [1,1])
-%!   assert (!isempty(fieldnames(nm)))
-%!   assert (sort(fieldnames(nm)),{'word1';'word2'})
-%!   assert (nm.word1,'short')
-%!   assert (nm.word2,'test')
-%! endif
+%! [nm, m, te, e, s, t] = regexp('short test string','(?<word1>\w*t)\s*(?<word2>\w*t)', 'names', 'match', 'tokenExtents', 'end', 'start', 'tokens');
+%! assert (s,1)
+%! assert (e,10)
+%! assert (size(te), [1,1])
+%! assert (te{1}, [1 5; 7, 10])
+%! assert (m{1},'short test')
+%! assert (size(t),[1,1])
+%! assert (t{1}{1},'short')
+%! assert (t{1}{2},'test')
+%! assert (size(nm), [1,1])
+%! assert (!isempty(fieldnames(nm)))
+%! assert (sort(fieldnames(nm)),{'word1';'word2'})
+%! assert (nm.word1,'short')
+%! assert (nm.word2,'test')
 
-%!test
+%!testif HAVE_PCRE
 %! ## This test is expected to fail if PCRE is not installed
-%! if (!isempty(findstr(octave_config_info ("DEFS"),"HAVE_PCRE")))
-%!   [nm, m, te, e, s, t] = regexp('short test string','(?<word1>\w*t)\s*(?<word2>\w*t)', 'names', 'match', 'tokenExtents', 'end', 'start', 'tokens');
-%!   assert (s,1)
-%!   assert (e,10)
-%!   assert (size(te), [1,1])
-%!   assert (te{1}, [1 5; 7, 10])
-%!   assert (m{1},'short test')
-%!   assert (size(t),[1,1])
-%!   assert (t{1}{1},'short')
-%!   assert (t{1}{2},'test')
-%!   assert (size(nm), [1,1])
-%!   assert (!isempty(fieldnames(nm)))
-%!   assert (sort(fieldnames(nm)),{'word1';'word2'})
-%!   assert (nm.word1,'short')
-%!   assert (nm.word2,'test')
-%! endif
-
-%!test
-%! ## This test is expected to fail if PCRE is not installed
-%! if (!isempty(findstr(octave_config_info ("DEFS"),"HAVE_PCRE")))
-%!   [t, nm] = regexp("John Davis\nRogers, James",'(?<first>\w+)\s+(?<last>\w+)|(?<last>\w+),\s+(?<first>\w+)','tokens','names');
-%!   assert (size(t), [1,2]);
-%!   assert (t{1}{1},'John');
-%!   assert (t{1}{2},'Davis');
-%!   assert (t{2}{1},'Rogers');
-%!   assert (t{2}{2},'James');
-%!   assert (size(nm), [1,1]);
-%!   assert (nm.first{1},'John');
-%!   assert (nm.first{2},'James');
-%!   assert (nm.last{1},'Davis');
-%!   assert (nm.last{2},'Rogers');
-%! endif
+%! [t, nm] = regexp("John Davis\nRogers, James",'(?<first>\w+)\s+(?<last>\w+)|(?<last>\w+),\s+(?<first>\w+)','tokens','names');
+%! assert (size(t), [1,2]);
+%! assert (t{1}{1},'John');
+%! assert (t{1}{2},'Davis');
+%! assert (t{2}{1},'Rogers');
+%! assert (t{2}{2},'James');
+%! assert (size(nm), [1,1]);
+%! assert (nm.first{1},'John');
+%! assert (nm.first{2},'James');
+%! assert (nm.last{1},'Davis');
+%! assert (nm.last{2},'Rogers');
 
 %!assert(regexp("abc\nabc",'.'),[1:7])
 %!assert(regexp("abc\nabc",'.','dotall'),[1:7])
-%!test
-%! if (!isempty(findstr(octave_config_info ("DEFS"),"HAVE_PCRE")))
-%!   assert(regexp("abc\nabc",'(?s).'),[1:7])
-%!   assert(regexp("abc\nabc",'.','dotexceptnewline'),[1,2,3,5,6,7])
-%!   assert(regexp("abc\nabc",'(?-s).'),[1,2,3,5,6,7])
-%! endif
+%!testif HAVE_PCRE
+%! assert(regexp("abc\nabc",'(?s).'),[1:7])
+%! assert(regexp("abc\nabc",'.','dotexceptnewline'),[1,2,3,5,6,7])
+%! assert(regexp("abc\nabc",'(?-s).'),[1,2,3,5,6,7])
 
 %!assert(regexp("caseCaSe",'case'),1)
 %!assert(regexp("caseCaSe",'case',"matchcase"),1)
 %!assert(regexp("caseCaSe",'case',"ignorecase"),[1,5])
-%!test
-%! if (!isempty(findstr(octave_config_info ("DEFS"),"HAVE_PCRE")))
-%!   assert(regexp("caseCaSe",'(?-i)case'),1)
-%!   assert(regexp("caseCaSe",'(?i)case'),[1,5])
-%! endif
+%!testif HAVE_PCRE
+%! assert(regexp("caseCaSe",'(?-i)case'),1)
+%! assert(regexp("caseCaSe",'(?i)case'),[1,5])
 
 %!assert (regexp("abc\nabc",'c$'),7)
 %!assert (regexp("abc\nabc",'c$',"stringanchors"),7)
-%!test
-%! if (!isempty(findstr(octave_config_info ("DEFS"),"HAVE_PCRE")))
-%!   assert (regexp("abc\nabc",'(?-m)c$'),7)
-%!   assert (regexp("abc\nabc",'c$',"lineanchors"),[3,7])
-%!   assert (regexp("abc\nabc",'(?m)c$'),[3,7])
-%! endif
+%!testif HAVE_PCRE
+%! assert (regexp("abc\nabc",'(?-m)c$'),7)
+%! assert (regexp("abc\nabc",'c$',"lineanchors"),[3,7])
+%! assert (regexp("abc\nabc",'(?m)c$'),[3,7])
 
 %!assert (regexp("this word",'s w'),4)
 %!assert (regexp("this word",'s w','literalspacing'),4)
-%!test
-%! if (!isempty(findstr(octave_config_info ("DEFS"),"HAVE_PCRE")))
-%!   assert (regexp("this word",'(?-x)s w','literalspacing'),4)
-%!   assert (regexp("this word",'s w','freespacing'),zeros(1,0))
-%!   assert (regexp("this word",'(?x)s w'),zeros(1,0))
-%! endif
+%!testif HAVE_PCRE
+%! assert (regexp("this word",'(?-x)s w','literalspacing'),4)
+%! assert (regexp("this word",'s w','freespacing'),zeros(1,0))
+%! assert (regexp("this word",'(?x)s w'),zeros(1,0))
 
 %!error regexp('string', 'tri', 'BadArg');
 %!error regexp('string');
@@ -1100,95 +1118,79 @@ if there are none. See @code{regexp} for more details\n\
 %! [s, e, te, m, t] = regexpi('ShoRt Test String','\w*r\w*','once');
 %! assert (s,1)
 %! assert (e,5)
-%! assert (size(te), [1,1])
-%! assert (isempty(te{1}))
-%! assert (m{1},'ShoRt')
-%! ## Matlab gives [1,0] here but that seems wrong.
-%! assert (size(t), [1,1])
+%! assert (isempty(te))
+%! assert (m,'ShoRt')
+%! assert (isempty(t))
 
 %!test
 %! [m, te, e, s, t] = regexpi('ShoRt Test String','\w*r\w*','once', 'match', 'tokenExtents', 'end', 'start', 'tokens');
 %! assert (s,1)
 %! assert (e,5)
+%! assert (isempty(te))
+%! assert (m,'ShoRt')
+%! assert (isempty(t))
+
+%!testif HAVE_PCRE
+%! ## This test is expected to fail if PCRE is not installed
+%! [s, e, te, m, t, nm] = regexpi('ShoRt Test String','(?<word1>\w*t)\s*(?<word2>\w*t)');
+%! assert (s,1)
+%! assert (e,10)
 %! assert (size(te), [1,1])
-%! assert (isempty(te{1}))
-%! assert (m{1},'ShoRt')
-%! ## Matlab gives [1,0] here but that seems wrong.
-%! assert (size(t), [1,1])
+%! assert (te{1}, [1 5; 7, 10])
+%! assert (m{1},'ShoRt Test')
+%! assert (size(t),[1,1])
+%! assert (t{1}{1},'ShoRt')
+%! assert (t{1}{2},'Test')
+%! assert (size(nm), [1,1])
+%! assert (!isempty(fieldnames(nm)))
+%! assert (sort(fieldnames(nm)),{'word1';'word2'})
+%! assert (nm.word1,'ShoRt')
+%! assert (nm.word2,'Test')
 
-%!test
+%!testif HAVE_PCRE
 %! ## This test is expected to fail if PCRE is not installed
-%! if (!isempty(findstr(octave_config_info ("DEFS"),"HAVE_PCRE")))
-%!   [s, e, te, m, t, nm] = regexpi('ShoRt Test String','(?<word1>\w*t)\s*(?<word2>\w*t)');
-%!   assert (s,1)
-%!   assert (e,10)
-%!   assert (size(te), [1,1])
-%!   assert (te{1}, [1 5; 7, 10])
-%!   assert (m{1},'ShoRt Test')
-%!   assert (size(t),[1,1])
-%!   assert (t{1}{1},'ShoRt')
-%!   assert (t{1}{2},'Test')
-%!   assert (size(nm), [1,1])
-%!   assert (!isempty(fieldnames(nm)))
-%!   assert (sort(fieldnames(nm)),{'word1';'word2'})
-%!   assert (nm.word1,'ShoRt')
-%!   assert (nm.word2,'Test')
-%! endif
-
-%!test
-%! ## This test is expected to fail if PCRE is not installed
-%! if (!isempty(findstr(octave_config_info ("DEFS"),"HAVE_PCRE")))
-%!   [nm, m, te, e, s, t] = regexpi('ShoRt Test String','(?<word1>\w*t)\s*(?<word2>\w*t)', 'names', 'match', 'tokenExtents', 'end', 'start', 'tokens');
-%!   assert (s,1)
-%!   assert (e,10)
-%!   assert (size(te), [1,1])
-%!   assert (te{1}, [1 5; 7, 10])
-%!   assert (m{1},'ShoRt Test')
-%!   assert (size(t),[1,1])
-%!   assert (t{1}{1},'ShoRt')
-%!   assert (t{1}{2},'Test')
-%!   assert (size(nm), [1,1])
-%!   assert (!isempty(fieldnames(nm)))
-%!   assert (sort(fieldnames(nm)),{'word1';'word2'})
-%!   assert (nm.word1,'ShoRt')
-%!   assert (nm.word2,'Test')
-%! endif
+%! [nm, m, te, e, s, t] = regexpi('ShoRt Test String','(?<word1>\w*t)\s*(?<word2>\w*t)', 'names', 'match', 'tokenExtents', 'end', 'start', 'tokens');
+%! assert (s,1)
+%! assert (e,10)
+%! assert (size(te), [1,1])
+%! assert (te{1}, [1 5; 7, 10])
+%! assert (m{1},'ShoRt Test')
+%! assert (size(t),[1,1])
+%! assert (t{1}{1},'ShoRt')
+%! assert (t{1}{2},'Test')
+%! assert (size(nm), [1,1])
+%! assert (!isempty(fieldnames(nm)))
+%! assert (sort(fieldnames(nm)),{'word1';'word2'})
+%! assert (nm.word1,'ShoRt')
+%! assert (nm.word2,'Test')
 
 %!assert(regexpi("abc\nabc",'.'),[1:7])
 %!assert(regexpi("abc\nabc",'.','dotall'),[1:7])
-%!test
-%! if (!isempty(findstr(octave_config_info ("DEFS"),"HAVE_PCRE")))
-%!   assert(regexpi("abc\nabc",'(?s).'),[1:7])
-%!   assert(regexpi("abc\nabc",'.','dotexceptnewline'),[1,2,3,5,6,7])
-%!   assert(regexpi("abc\nabc",'(?-s).'),[1,2,3,5,6,7])
-%! endif
+%!testif HAVE_PCRE
+%! assert(regexpi("abc\nabc",'(?s).'),[1:7])
+%! assert(regexpi("abc\nabc",'.','dotexceptnewline'),[1,2,3,5,6,7])
+%! assert(regexpi("abc\nabc",'(?-s).'),[1,2,3,5,6,7])
 
 %!assert(regexpi("caseCaSe",'case'),[1,5])
 %!assert(regexpi("caseCaSe",'case',"matchcase"),1)
 %!assert(regexpi("caseCaSe",'case',"ignorecase"),[1,5])
-%!test
-%! if (!isempty(findstr(octave_config_info ("DEFS"),"HAVE_PCRE")))
-%!   assert(regexpi("caseCaSe",'(?-i)case'),1)
-%!   assert(regexpi("caseCaSe",'(?i)case'),[1,5])
-%! endif
+%!testif HAVE_PCRE
+%! assert(regexpi("caseCaSe",'(?-i)case'),1)
+%! assert(regexpi("caseCaSe",'(?i)case'),[1,5])
 
 %!assert (regexpi("abc\nabc",'c$'),7)
 %!assert (regexpi("abc\nabc",'c$',"stringanchors"),7)
-%!test
-%! if (!isempty(findstr(octave_config_info ("DEFS"),"HAVE_PCRE")))
-%!   assert (regexpi("abc\nabc",'(?-m)c$'),7)
-%!   assert (regexpi("abc\nabc",'c$',"lineanchors"),[3,7])
-%!   assert (regexpi("abc\nabc",'(?m)c$'),[3,7])
-%! endif
+%!testif HAVE_PCRE
+%! assert (regexpi("abc\nabc",'(?-m)c$'),7)
+%! assert (regexpi("abc\nabc",'c$',"lineanchors"),[3,7])
+%! assert (regexpi("abc\nabc",'(?m)c$'),[3,7])
 
 %!assert (regexpi("this word",'s w'),4)
 %!assert (regexpi("this word",'s w','literalspacing'),4)
-%!test
-%! if (!isempty(findstr(octave_config_info ("DEFS"),"HAVE_PCRE")))
-%!   assert (regexpi("this word",'(?-x)s w','literalspacing'),4)
-%!   assert (regexpi("this word",'s w','freespacing'),zeros(1,0))
-%!   assert (regexpi("this word",'(?x)s w'),zeros(1,0))
-%! endif
+%!testif HAVE_PCRE
+%! assert (regexpi("this word",'(?-x)s w','literalspacing'),4)
+%! assert (regexpi("this word",'s w','freespacing'),zeros(1,0))
+%! assert (regexpi("this word",'(?x)s w'),zeros(1,0))
 
 %!error regexpi('string', 'tri', 'BadArg');
 %!error regexpi('string');
@@ -1262,7 +1264,8 @@ octregexprep (const octave_value_list &args, const std::string &nm)
       std::list<regexp_elem> lst;
       string_vector named;
       int nopts;
-      int sz = octregexp_list (regexpargs, nm , false, lst, named, nopts);
+      bool once;
+      int sz = octregexp_list (regexpargs, nm , false, lst, named, nopts, once);
 
       if (error_state)
 	return retval;
@@ -1348,7 +1351,8 @@ octregexprep (const octave_value_list &args, const std::string &nm)
       std::list<regexp_elem> lst;
       string_vector named;
       int nopts;
-      int sz = octregexp_list (regexpargs, nm, false, lst, named,nopts);
+      bool once;
+      int sz = octregexp_list (regexpargs, nm, false, lst, named, nopts, once);
 
       if (error_state)
 	return retval;
@@ -1531,13 +1535,11 @@ Alternatively, use (?x) or (?-x) in the pattern.\n\
 %! t = regexprep(xml,'<[!?][^>]*>','','tokenize');
 %! assert(t,' <tag v="hello">some stuff</tag>')
 
-%!test  # Capture replacement
-%! if (!isempty(findstr(octave_config_info ("DEFS"),"HAVE_PCRE")))
-%!   data = "Bob Smith\nDavid Hollerith\nSam Jenkins";
-%!   result = "Smith, Bob\nHollerith, David\nJenkins, Sam";
-%!   t = regexprep(data,'(?m)^(\w+)\s+(\w+)$','$2, $1');
-%!   assert(t,result)
-%! end
+%!testif HAVE_PCRE # Capture replacement
+%! data = "Bob Smith\nDavid Hollerith\nSam Jenkins";
+%! result = "Smith, Bob\nHollerith, David\nJenkins, Sam";
+%! t = regexprep(data,'(?m)^(\w+)\s+(\w+)$','$2, $1');
+%! assert(t,result)
 
 # Return the original if no match
 %!assert(regexprep('hello','world','earth'),'hello')
