@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1996, 1997, 1998, 2000, 2001, 2002, 2003, 2004, 2005,
+              2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -176,16 +176,10 @@ octave_complex::resize (const dim_vector& dv, bool fill) const
 }
 
 bool 
-octave_complex::save_ascii (std::ostream& os, bool& infnan_warned)
+octave_complex::save_ascii (std::ostream& os)
 {
   Complex c = complex_value ();
 
-  if (! infnan_warned && (xisnan (c) || xisinf (c)))
-    {
-      warning ("save: Inf or NaN values may not be reloadable");
-      infnan_warned = true;
-    }
-      
   octave_write_complex (os, c);
 
   os << "\n";
