@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+              2002, 2003, 2004, 2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -32,6 +32,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 #include <iomanip>
 #include <fstream>
+#include <sstream>
 
 #include "glob-match.h"
 #include "str-vec.h"
@@ -44,6 +45,8 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "symtab.h"
 #include "utils.h"
 #include "variables.h"
+#include "ov-usr-fcn.h"
+#include "toplev.h"
 
 #include "gripes.h"
 #include "lo-mappers.h"
@@ -55,13 +58,14 @@ unsigned long int symbol_table::symtab_count = 0;
 // Should variables be allowed to hide functions of the same name?  A
 // positive value means yes.  A negative value means yes, but print a
 // warning message.  Zero means it should be considered an error.
-static int Vvariables_can_hide_functions;
+static int Vvariables_can_hide_functions = 1;
 
 // Nonzero means we print debugging info about symbol table lookups.
-static int Vdebug_symtab_lookups;
+static bool Vdebug_symtab_lookups = false;
 
 // Defines layout for the whos/who -long command
-std::string Vwhos_line_format;
+std::string Vwhos_line_format
+  = "  %p:4; %ln:6; %cs:16:6:8:1;  %rb:12;  %lc:-1;\n";
 
 octave_allocator
 symbol_record::symbol_def::allocator (sizeof (symbol_record::symbol_def));
@@ -81,14 +85,12 @@ SYMBOL_DEF::type_as_string (void) const
     retval = "built-in mapper function";
   else if (is_user_function ())
     retval = "user-defined function";
-  else if (is_builtin_constant ())
-    retval = "built-in constant";
-  else if (is_builtin_variable ())
-    retval = "built-in variable";
   else if (is_builtin_function ())
     retval = "built-in function";
   else if (is_dld_function ())
     retval = "dynamically-linked function";
+  else if (is_mex_function ())
+    retval = "dynamically-linked mex function";
 
   return retval;
 }
@@ -131,9 +133,7 @@ SYMBOL_DEF::type (std::ostream& os, const std::string& name, bool pr_type_info,
 	  defn->accept (tpc);
 	}
     }
-  else if (is_user_variable ()
-	   || is_builtin_variable ()
-	   || is_builtin_constant ())
+  else if (is_user_variable ())
     {
       if (pr_type_info && ! quiet)
 	os << name << " is a " << type_as_string () << "\n";
@@ -152,7 +152,7 @@ SYMBOL_DEF::which (const std::string& name)
 {
   std::string retval;
 
-  if (is_user_function () || is_dld_function ())
+  if (is_user_function () || is_dld_function () || is_mex_function ())
     {
       octave_function *defn = definition.function_value ();
 
@@ -170,7 +170,7 @@ SYMBOL_DEF::which (std::ostream& os, const std::string& name)
 {
   os << name;
 
-  if (is_user_function () || is_dld_function ())
+  if (is_user_function () || is_dld_function () || is_mex_function ())
     {
       octave_function *defn = definition.function_value ();
 
@@ -188,6 +188,21 @@ SYMBOL_DEF::which (std::ostream& os, const std::string& name)
   os << " is a " << type_as_string () << "\n";
 }
 
+void
+SYMBOL_DEF::document (const std::string& h)
+{
+  help_string = h;
+
+  if (is_function ())
+    {
+      octave_function *defn = definition.function_value ();
+
+      if (defn)
+	defn->document (h);
+    }
+}
+
+
 void
 SYMBOL_DEF::print_info (std::ostream& os, const std::string& prefix) const
 {
@@ -209,36 +224,7 @@ void
 symbol_record::define (const octave_value& v, unsigned int sym_type)
 {
   if (! (is_variable () && read_only_error ("redefine")))
-    {
-      if (definition->type () == symbol_record::BUILTIN_VARIABLE)
-	sym_type = symbol_record::BUILTIN_VARIABLE;
-
-      definition->define (v, sym_type);
-    }
-}
-
-void
-symbol_record::define_builtin_var (const octave_value& v)
-{
-  define (v, symbol_record::BUILTIN_VARIABLE);
-
-  if (chg_fcn)
-    chg_fcn ();
-}
-
-bool
-symbol_record::define_builtin_const (const octave_value& v)
-{
-  bool retval = false;
-
-  if (! read_only_error ("redefine"))
-    {
-      definition->define (v, symbol_record::BUILTIN_CONSTANT);
-
-      retval = true;
-    }
-
-  return retval;
+    definition->define (v, sym_type);
 }
 
 bool
@@ -248,12 +234,11 @@ symbol_record::define (octave_function *f, unsigned int sym_type)
 
   if (! read_only_error ("redefine"))
     {
+      maybe_delete_def ();
+
       octave_value tmp (f);
 
-      if (! definition)
-	definition = new symbol_def (tmp, sym_type);
-      else
-	definition->define (tmp, sym_type);
+      definition = new symbol_def (tmp, sym_type);
 
       retval = true;
     }
@@ -266,10 +251,16 @@ symbol_record::clear (void)
 {
   if (is_defined ())
     {
-      if (! tagged_static)
+      if (! (tagged_static || is_eternal ()))
 	{
-	  if (--definition->count <= 0)
-	    delete definition;
+	  while (! aliases_to_clear.empty ())
+	    {
+	      symbol_record *sr = aliases_to_clear.top ();
+	      aliases_to_clear.pop ();
+	      sr->clear ();
+	    }
+
+	  maybe_delete_def ();
 
 	  definition = new symbol_def ();
 	}
@@ -280,14 +271,16 @@ symbol_record::clear (void)
 }
 
 void
-symbol_record::alias (symbol_record *s)
+symbol_record::alias (symbol_record *s, bool mark_to_clear)
 {
   chg_fcn = s->chg_fcn;
 
-  if (--definition->count <= 0)
-    delete definition;
+  maybe_delete_def ();
 
-  definition = (s->definition);
+  if (mark_to_clear)
+    s->push_alias_to_clear (this);
+
+  definition = s->definition;
 
   definition->count++;
 }
@@ -299,12 +292,25 @@ symbol_record::mark_as_formal_parameter (void)
     error ("can't mark global variable `%s' as function parameter",
 	   nm.c_str ());
   else if (is_static ())
-    error ("can't mark static variable `%s' as function paraemter",
+    error ("can't mark static variable `%s' as function parameter",
 	   nm.c_str ());
   else
     formal_param = 1;
 }
 
+void
+symbol_record::mark_as_automatic_variable (void)
+{
+  if (is_linked_to_global ())
+    error ("can't mark global variable `%s' as automatic variable",
+	   nm.c_str ());
+  else if (is_static ())
+    error ("can't mark static variable `%s' as automatic variable",
+	   nm.c_str ());
+  else
+    automatic_variable = 1;
+}
+
 void
 symbol_record::mark_as_linked_to_global (void)
 {
@@ -335,15 +341,6 @@ symbol_record::variable_value (void)
   return is_variable () ? def () : foo;
 }
 
-inline void
-symbol_record::link_to_builtin_variable (void)
-{
-  symbol_record *tmp_sym = fbi_sym_tab->lookup (name ());
-
-  if (tmp_sym && tmp_sym->is_builtin_variable ())
-    alias (tmp_sym);
-}
-
 octave_lvalue
 symbol_record::variable_reference (void)
 {
@@ -360,19 +357,13 @@ symbol_record::variable_reference (void)
 	}
     }
 
-  if (is_function () || is_constant ())
+  if (is_function ())
     clear ();
 
   if (! is_defined ())
     {
-      if (! (is_formal_parameter () || is_linked_to_global ()))
-	link_to_builtin_variable ();
-
-      if (! is_defined ())
-	{
-	  octave_value tmp;
-	  define (tmp);
-	}
+      octave_value tmp;
+      define (tmp);
     }
 
   return octave_lvalue (&(def ()), chg_fcn);
@@ -402,8 +393,7 @@ symbol_record::pop_context (void)
 
   if (! context.empty ())
     {
-      if (--definition->count <= 0)
-	delete definition;
+      maybe_delete_def ();
 
       definition = context.top ();
       context.pop ();
@@ -462,13 +452,13 @@ symbol_record::dimensions_string_req_first_space (int print_dims) const
   return first_param_space;
 }
 
-// Calculate how much space needs to be reserved for the the
+// Calculate how much space needs to be reserved for the
 // dimensions string.  For example,
 //
 //   mat is a 12x3 matrix
 //            ^^^^ => 4 columns
 //
-// XXX FIXME XXX -- why not just use the dim_vector::str () method?
+// FIXME -- why not just use the dim_vector::str () method?
 
 int
 symbol_record::dimensions_string_req_total_space (int print_dims) const
@@ -485,7 +475,7 @@ symbol_record::dimensions_string_req_total_space (int print_dims) const
 // Make the dimensions-string.  For example: mat is a 2x3 matrix.
 //                                                    ^^^
 //
-// XXX FIXME XXX -- why not just use the dim_vector::str () method?
+// FIXME -- why not just use the dim_vector::str () method?
 
 std::string
 symbol_record::make_dimensions_string (int print_dims) const
@@ -607,6 +597,10 @@ symbol_record::print_symbol_info_line (std::ostream& os,
 	      os << byte_size ();
 	      break;
 
+	    case 'c':
+	      os << class_name ();
+	      break;
+
 	    case 'e':
 	      os << capacity ();
 	      break;
@@ -656,6 +650,12 @@ symbol_record::print_symbol_info_line (std::ostream& os,
 void
 symbol_record::print_info (std::ostream& os, const std::string& prefix) const
 {
+  os << prefix << "formal param:      " << formal_param << "\n"
+     << prefix << "linked to global:  " << linked_to_global << "\n"
+     << prefix << "tagged static:     " << tagged_static << "\n"
+     << prefix << "can hide function: " << can_hide_function << "\n"
+     << prefix << "visible:           " << visible << "\n";
+
   if (definition)
     definition->print_info (os, prefix);
   else
@@ -667,7 +667,7 @@ symbol_record::read_only_error (const char *action)
 {
   if (is_read_only ())
     {
-      if (is_variable () || is_constant ())
+      if (is_variable ())
 	::error ("can't %s read-only constant `%s'", action, nm.c_str ());
       else if (is_function ())
 	::error ("can't %s read-only function `%s'", action, nm.c_str ());
@@ -784,7 +784,7 @@ symbol_table::rename (const std::string& old_name, const std::string& new_name)
 	 new_name.c_str ());
 }
 
-// XXX FIXME XXX -- it would be nice to eliminate a lot of the
+// FIXME -- it would be nice to eliminate a lot of the
 // following duplicate code.
 
 void
@@ -796,6 +796,32 @@ symbol_table::clear (void)
 
       while (ptr)
 	{
+	  if (ptr->is_user_function())
+	    {
+	      octave_user_function *fcn = ptr->def ().user_function_value ();
+	      std::string parent = (fcn ? fcn->parent_fcn_name () :
+				    std::string ());
+
+	      if (! parent.empty ())
+		{
+		  if (curr_parent_function &&
+		      parent == curr_parent_function->name ())
+		    {
+		      ptr = ptr->next ();
+		      continue;
+		    }			  
+
+		  symbol_record *parent_sr = fbi_sym_tab->lookup (parent);
+
+		  if (parent_sr && (parent_sr->is_static () ||
+				    parent_sr->is_eternal ()))
+		  {
+		    ptr = ptr->next ();
+		    continue;
+		  }
+		}
+	    }
+
 	  ptr->clear ();
 
 	  ptr = ptr->next ();
@@ -831,7 +857,56 @@ symbol_table::clear_functions (void)
 
       while (ptr)
 	{
-	  if (ptr->is_user_function () || ptr->is_dld_function ())
+	  if (ptr->is_user_function ()
+	      || ptr->is_dld_function ()
+	      || ptr->is_mex_function ())
+	    {
+
+	      if (ptr->is_user_function())
+		{
+		  octave_user_function *fcn = 
+		    ptr->def ().user_function_value ();
+		  std::string parent = (fcn ? fcn->parent_fcn_name () :
+					std::string ());
+
+		  if (! parent.empty ())
+		    {
+		      if (curr_parent_function &&
+			  parent == curr_parent_function->name ())
+			{
+			  ptr = ptr->next ();
+			  continue;
+			}			  
+
+		      symbol_record *parent_sr = fbi_sym_tab->lookup (parent);
+
+		      if (parent_sr && (parent_sr->is_static () ||
+					parent_sr->is_eternal ()))
+			{
+			  ptr = ptr->next ();
+			  continue;
+			}
+		    }
+		}
+
+	      ptr->clear ();
+	    }
+
+	  ptr = ptr->next ();
+	}
+    }
+}
+
+void
+symbol_table::clear_mex_functions (void)
+{
+  for (unsigned int i = 0; i < table_size; i++)
+    {
+      symbol_record *ptr = table[i].next ();
+
+      while (ptr)
+	{
+	  if (ptr->is_mex_function ())
 	    ptr->clear ();
 
 	  ptr = ptr->next ();
@@ -867,7 +942,29 @@ symbol_table::clear (const std::string& nm)
     {
       if (ptr->name () == nm)
 	{
+	  if (ptr->is_user_function())
+	    {
+	      octave_user_function *fcn = 
+		ptr->def ().user_function_value ();
+	      std::string parent = (fcn ? fcn->parent_fcn_name () :
+				    std::string ());
+
+	      if (! parent.empty ())
+		{
+		  if (curr_parent_function &&
+		      parent == curr_parent_function->name ())
+		    return true;
+
+		  symbol_record *parent_sr = fbi_sym_tab->lookup (parent);
+		  
+		  if (parent_sr && (parent_sr->is_static () ||
+				    parent_sr->is_eternal ()))
+		    return true;
+		}
+	    }
+
 	  ptr->clear ();
+
 	  return true;
 	}
       ptr = ptr->next ();
@@ -930,8 +1027,31 @@ symbol_table::clear_function (const std::string& nm)
   while (ptr)
     {
       if (ptr->name () == nm
-	  && (ptr->is_user_function () || ptr->is_dld_function ()))
+	  && (ptr->is_user_function ()
+	      || ptr->is_dld_function ()
+	      || ptr->is_mex_function ()))
 	{
+	  if (ptr->is_user_function())
+	    {
+	      octave_user_function *fcn = 
+		ptr->def ().user_function_value ();
+	      std::string parent = (fcn ? fcn->parent_fcn_name () :
+				    std::string ());
+
+	      if (! parent.empty ())
+		{
+		  if (curr_parent_function &&
+		      parent == curr_parent_function->name ())
+		    return true;
+
+		  symbol_record *parent_sr = fbi_sym_tab->lookup (parent);
+		  
+		  if (parent_sr && (parent_sr->is_static () ||
+				    parent_sr->is_eternal ()))
+		    return true;
+		}
+	    }
+
 	  ptr->clear ();
 	  return true;
 	}
@@ -1014,7 +1134,9 @@ symbol_table::clear_function_pattern (const std::string& pat)
 
       while (ptr)
 	{
-	  if (ptr->is_user_function () || ptr->is_dld_function ())
+	  if (ptr->is_user_function ()
+	      || ptr->is_dld_function ()
+	      || ptr->is_mex_function ())
 	    {
 	      glob_match pattern (pat);
 
@@ -1073,7 +1195,7 @@ symbol_table::subsymbol_list (const string_vector& pats,
 
   int n = size ();
 
-  Array<symbol_record *> subsymbols (n);
+  Array<symbol_record *> subsymbols (dim_vector (n, 1));
   int pats_length = pats.length ();
 
   if (n == 0)
@@ -1123,7 +1245,7 @@ symbol_table::subsymbol_list (const string_vector& pats,
 	}
     }
 
-  subsymbols.resize (count);
+  subsymbols.resize (dim_vector (count, 1));
 
   return subsymbols;
 }
@@ -1136,7 +1258,7 @@ symbol_table::symbol_list (const string_vector& pats,
 
   int n = size ();
 
-  Array<symbol_record *> symbols (n);
+  Array<symbol_record *> symbols (dim_vector (n, 1));
 
   if (n == 0)
     return symbols;
@@ -1147,22 +1269,25 @@ symbol_table::symbol_list (const string_vector& pats,
 
       while (ptr)
 	{
-	  assert (count < n);
+	  if (ptr->is_visible ())
+	    {
+	      assert (count < n);
 
-	  unsigned int my_scope = ptr->is_linked_to_global () + 1; // Tricky...
+	      unsigned int my_scope = ptr->is_linked_to_global () + 1; // Tricky...
 
-	  unsigned int my_type = ptr->type ();
+	      unsigned int my_type = ptr->type ();
 
-	  std::string my_name = ptr->name ();
+	      std::string my_name = ptr->name ();
 
-	  if ((type & my_type) && (scope & my_scope) && (matches_patterns (my_name, pats)))
-	    symbols(count++) = ptr;
+	      if ((type & my_type) && (scope & my_scope) && (matches_patterns (my_name, pats)))
+		symbols(count++) = ptr;
+	    }
 
 	  ptr = ptr->next ();
 	}
     }
 
-  symbols.resize (count);
+  symbols.resize (dim_vector (count, 1));
 
   return symbols;
 }
@@ -1194,8 +1319,8 @@ symbol_table::name_list (const string_vector& pats, bool sort,
 static int
 maybe_list_cmp_fcn (const void *a_arg, const void *b_arg)
 {
-  const symbol_record *a = *(X_CAST (const symbol_record **, a_arg));
-  const symbol_record *b = *(X_CAST (const symbol_record **, b_arg));
+  const symbol_record *a = *(static_cast<symbol_record *const*> (a_arg));
+  const symbol_record *b = *(static_cast<symbol_record *const*> (b_arg));
 
   std::string a_nm = a->name ();
   std::string b_nm = b->name ();
@@ -1209,7 +1334,7 @@ symbol_table::print_descriptor (std::ostream& os,
 {
   // This method prints a line of information on a given symbol
   std::list<whos_parameter>::iterator i = params.begin ();
-  OSSTREAM param_buf;
+  std::ostringstream param_buf;
 
   while (i != params.end ())
     {
@@ -1286,9 +1411,7 @@ symbol_table::print_descriptor (std::ostream& os,
 	}
     }
 
-  param_buf << OSSTREAM_ENDS;
-  os << OSSTREAM_C_STR (param_buf);
-  OSSTREAM_FREEZE (param_buf);
+  os << param_buf.str ();
 }
 
 std::list<whos_parameter>
@@ -1305,33 +1428,35 @@ symbol_table::parse_whos_line_format (Array<symbol_record *>& symbols) const
   size_t bytes1;
   int elements1;
 
-  int len = symbols.length (), i;
+  int len = symbols.length ();
 
-  std::string param_string = "benpst";
-  Array<int> param_length(param_string.length ());
-  Array<std::string> param_names(param_string.length ());
-  size_t pos_b, pos_t, pos_e, pos_n, pos_p, pos_s;
+  std::string param_string = "bcenpst";
+  Array<int> param_length (dim_vector (param_string.length (), 1));
+  Array<std::string> param_names (dim_vector (param_string.length (), 1));
+  size_t pos_b, pos_c, pos_e, pos_n, pos_p, pos_s, pos_t;
 
   pos_b = param_string.find ('b'); // Bytes
-  pos_t = param_string.find ('t'); // (Type aka) Class
+  pos_c = param_string.find ('c'); // Class
   pos_e = param_string.find ('e'); // Elements
   pos_n = param_string.find ('n'); // Name
   pos_p = param_string.find ('p'); // Protected
   pos_s = param_string.find ('s'); // Size
+  pos_t = param_string.find ('t'); // Type
 
   param_names(pos_b) = "Bytes";
-  param_names(pos_t) = "Class";
+  param_names(pos_c) = "Class";
   param_names(pos_e) = "Elements";
   param_names(pos_n) = "Name";
   param_names(pos_p) = "Prot";
   param_names(pos_s) = "Size";
+  param_names(pos_t) = "Type";
 
-  for (i = 0; i < 6; i++)
+  for (size_t i = 0; i < param_string.length (); i++)
     param_length(i) = param_names(i) . length ();
 
   // Calculating necessary spacing for name column,
   // bytes column, elements column and class column
-  for (i = 0; i < static_cast<int> (len); i++)
+  for (int i = 0; i < static_cast<int> (len); i++)
     {
       std::stringstream ss1, ss2;
       std::string str;
@@ -1389,7 +1514,7 @@ symbol_table::parse_whos_line_format (Array<symbol_record *>& symbols) const
 
 	  idx += cmd.length ();
 
-	  // XXX FIXME XXX -- use iostream functions instead of sscanf!
+	  // FIXME -- use iostream functions instead of sscanf!
 
 	  if (cmd.find_first_of ("crl") != 1)
 	    items = sscanf (cmd.c_str (), "%c%c:%d:%d:%d:%d;",
@@ -1524,14 +1649,14 @@ symbol_table::maybe_list (const char *header, const string_vector& argv,
 
   if (show_verbose)
     {
-      // XXX FIXME XXX Should separate argv to lists with and without dots.
+      // FIXME Should separate argv to lists with and without dots.
       Array<symbol_record *> xsymbols = symbol_list (argv, type, scope);
       Array<symbol_record *> xsubsymbols = subsymbol_list (argv, type, scope);
 
       int sym_len = xsymbols.length (), subsym_len = xsubsymbols.length (),
 	len = sym_len + subsym_len;
  
-      Array<symbol_record *> symbols (len);
+      Array<symbol_record *> symbols (dim_vector (len, 1));
 
       if (len > 0)
 	{
@@ -1600,7 +1725,7 @@ symbol_table::glob (const std::string& pat, unsigned int type,
 
   int n = size ();
 
-  Array<symbol_record *> symbols (n);
+  Array<symbol_record *> symbols (dim_vector (n, 1));
 
   if (n == 0)
     return symbols;
@@ -1629,7 +1754,7 @@ symbol_table::glob (const std::string& pat, unsigned int type,
 	}
     }
 
-  symbols.resize (count);
+  symbols.resize (dim_vector (count, 1));
 
   return symbols;
 }
@@ -1664,6 +1789,73 @@ symbol_table::pop_context (void)
     }
 }
 
+// Create a new symbol table with the same entries.  Only the symbol
+// names and some attributes are copied, not values.
+
+symbol_table *
+symbol_table::dup (void)
+{
+  symbol_table *new_sym_tab = new symbol_table (table_size);
+
+  for (unsigned int i = 0; i < table_size; i++)
+    {
+      symbol_record *ptr = table[i].next ();
+
+      while (ptr)
+	{
+	  std::string nm = ptr->name ();
+
+	  symbol_record *sr = new_sym_tab->lookup (nm, true);
+
+	  if (sr)
+	    {
+	      if (ptr->is_formal_parameter ())
+		sr->mark_as_formal_parameter ();
+
+	      if (ptr->is_automatic_variable ())
+		sr->mark_as_automatic_variable ();
+
+	      if (ptr->is_static ())
+		sr->mark_as_static ();
+	    }
+
+	  ptr = ptr->next ();
+	}
+    }
+
+  return new_sym_tab;
+}
+
+void
+symbol_table::inherit (symbol_table *parent_sym_tab)
+{
+  for (unsigned int i = 0; i < table_size; i++)
+    {
+      symbol_record *ptr = table[i].next ();
+
+      while (ptr)
+	{
+	  std::string nm = ptr->name ();
+
+	  if (! (nm == "__retval__"
+		 || ptr->is_automatic_variable ()
+		 || ptr->is_formal_parameter ()))
+	    {
+	      symbol_record *sr = parent_sym_tab->lookup (nm);
+
+	      if (sr)
+		{
+		  ptr->define (sr->variable_value ());
+
+		  ptr->mark_as_static ();
+		}
+	    }
+
+	  ptr = ptr->next ();
+	}
+    }
+}
+
 void
 symbol_table::print_info (std::ostream& os) const
 {
@@ -1729,64 +1921,35 @@ symbol_table::hash (const std::string& str)
   return h & (table_size - 1);
 }
 
-
-static int
-variables_can_hide_functions (void)
+DEFUN (debug_symtab_lookups, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} debug_symtab_lookups ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} debug_symtab_lookups (@var{new_val})\n\
+Query or set the internal variable that controls whether debugging\n\
+information is printed when searching for symbols in the symbol tables.\n\
+@end deftypefn")
 {
-  Vvariables_can_hide_functions
-    = check_preference ("variables_can_hide_functions");
-
-  return 0;
+  return SET_INTERNAL_VARIABLE (debug_symtab_lookups);
 }
 
-static int
-debug_symtab_lookups (void)
-{
-  Vdebug_symtab_lookups = check_preference ("debug_symtab_lookups");
-
-  return 0;
-}
-
-static int
-whos_line_format (void)
-{
-  Vwhos_line_format = builtin_string_variable ("whos_line_format");
-
-  return 0;
-}
-
-void
-symbols_of_symtab (void)
-{
-  DEFVAR (variables_can_hide_functions, true, variables_can_hide_functions,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} variables_can_hide_functions\n\
-If the value of this variable is nonzero, assignments to variables may\n\
-hide previously defined functions of the same name.  A negative value\n\
-will cause Octave to print a warning, but allow the operation.\n\
-@end defvr");
-
-  DEFVAR (debug_symtab_lookups, false, debug_symtab_lookups,
-    "-*- texinfo -*-\n\
-@defvr debug_symtab_lookups\n\
-If the value of this variable is nonzero, print debugging info when\n\
-searching for symbols in the symbol tables.\n\
-@end defvr");
-
-  DEFVAR (whos_line_format, "  %p:4; %ln:6; %cs:16:6:8:1;  %rb:12;  %lt:-1;\n", whos_line_format,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} whos_line_format\n\
-This string decides in what order attributtes of variables are to be printed.\n\
-The following commands are used:\n\
+DEFUN (whos_line_format, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} whos_line_format ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} whos_line_format (@var{new_val})\n\
+Query or set the format string used by the @code{whos}.\n\
+\n\
+The following escape sequences may be used in the format:\n\
 @table @code\n\
 @item %b\n\
 Prints number of bytes occupied by variables.\n\
+@item %c\n\
+Prints class names of variables.\n\
 @item %e\n\
 Prints elements held by variables.\n\
 @item %n\n\
 Prints variable names.\n\
 @item %p\n\
-Prints protection attributtes of variables.\n\
+Prints protection attributes of variables.\n\
 @item %s\n\
 Prints dimensions of variables.\n\
 @item %t\n\
@@ -1804,7 +1967,11 @@ Centered (may only be applied to command %s).\n\
 @end table\n\
 \n\
 A command is composed like this:\n\
-%[modifier]<command>[:size_of_parameter[:center-specific[:print_dims[:balance]]]];\n\
+\n\
+@example\n\
+%[modifier]<command>[:size_of_parameter[:center-specific[\n\
+       :print_dims[:balance]]]];\n\
+@end example\n\
 \n\
 Command and modifier is already explained. Size_of_parameter\n\
 tells how many columns the parameter will need for printing.\n\
@@ -1816,8 +1983,24 @@ center-specific and print_dims may only be applied to command\n\
 dimensions whatsoever.\n\
 balance specifies the offset for printing of the dimensions string.\n\
 \n\
-Default format is \"  %p:4; %ln:6; %cs:16:6:8:1;  %rb:12;  %lt:-1;\\n\".\n\
-@end defvr\n");
+The default format is \"  %p:4; %ln:6; %cs:16:6:8:1;  %rb:12;  %lc:-1;\\n\".\n\
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE (whos_line_format);
+}
+
+DEFUN (variables_can_hide_functions, args, nargout,
+    "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} variables_can_hide_functions ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} variables_can_hide_functions (@var{new_val})\n\
+Query or set the internal variable that controls whether assignments\n\
+to variables may hide previously defined functions of the same name.\n\
+If set to a nonzero value allows hiding, zero causes Octave to\n\
+generate an error, and a negative value cause Octave to print a\n\
+warning, but allow the operation.\n\
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE (variables_can_hide_functions);
 }
 
 /*
