@@ -175,7 +175,7 @@ octave_call_stack::unwind_pop_script (void *)
   delete f;
 }
 
-static void
+void
 recover_from_exception (void)
 {
   can_interrupt = true;
@@ -805,7 +805,6 @@ specified option.\n\
       { false, "CXX_VERSION", OCTAVE_CONF_CXX_VERSION },
       { false, "DEFAULT_PAGER", OCTAVE_DEFAULT_PAGER },
       { false, "DEFS", OCTAVE_CONF_DEFS },
-      { false, "DLFCN_INCFLAGS", OCTAVE_CONF_DLFCN_INCFLAGS },
       { false, "DL_LD", OCTAVE_CONF_DL_LD },
       { false, "DL_LDFLAGS", OCTAVE_CONF_DL_LDFLAGS },
       { false, "ENABLE_DYNAMIC_LINKING", OCTAVE_CONF_ENABLE_DYNAMIC_LINKING },
@@ -828,7 +827,6 @@ specified option.\n\
       { false, "LEXLIB", OCTAVE_CONF_LEXLIB },
       { false, "LFLAGS", OCTAVE_CONF_LFLAGS },
       { false, "LIBCRUFT", OCTAVE_CONF_LIBCRUFT },
-      { false, "LIBDLFCN", OCTAVE_CONF_LIBDLFCN },
       { false, "LIBEXT", OCTAVE_CONF_LIBEXT },
       { false, "LIBFLAGS", OCTAVE_CONF_LIBFLAGS },
       { false, "LIBOCTAVE", OCTAVE_CONF_LIBOCTAVE },
